/*
 * Created by Julian Mutter on 7/10/18 3:58 PM
 * Copyright (c) 2018. All rights reserved.
 * Last modified 7/10/18 3:54 PM
 *
 */

package de.frajul.endlessroll.entities.collision;

import de.frajul.endlessroll.data.Vector;
import de.frajul.endlessroll.entities.collision.geometry.Circle;
import de.frajul.endlessroll.entities.collision.geometry.Triangle;

/**
 * Created by Julian on 28.02.2016.
 */
public class CircleTriangleCollisionDetector {

    public boolean isCircleTriangleCollision(Circle circle, Triangle triangle) {
        if (circleIntersectingWithTriangleVertices(circle, triangle))
            return true;

        if (circleCenterInsideTriangle(circle, triangle))
            return true;

        return circleIntersectingWithTriangleEdges(circle, triangle);
    }

    private boolean circleIntersectingWithTriangleEdges(Circle circle, Triangle triangle) {
        Vector edge1 = triangle.getBottomRightVertex().vectorTo(triangle.getTopRightVertex());
        Vector edge2 = triangle.getTopRightVertex().vectorTo(triangle.getBottomLeftVertex());
        Vector edge3 = triangle.getBottomLeftVertex().vectorTo(triangle.getBottomRightVertex());

        boolean intersectingWithEdge1 = circleIntersectingWithTriangleEdge(circle, triangle.getBottomRightVertex(), edge1);
        boolean intersectingWithEdge2 = circleIntersectingWithTriangleEdge(circle, triangle.getTopRightVertex(), edge2);
        boolean intersectingWithEdge3 = circleIntersectingWithTriangleEdge(circle, triangle.getBottomLeftVertex(), edge3);
        return intersectingWithEdge1 || intersectingWithEdge2 || intersectingWithEdge3;
    }

    private boolean circleIntersectingWithTriangleEdge(Circle circle, Vector vertex, Vector edge) {
        Vector vertexToCenter = vertex.vectorTo(circle.getPosition());
        Vector kVector = new Vector(vertexToCenter).mul(edge);
        float k = kVector.x + kVector.y;

        if (k > 0) {
            float length = edge.length();
            k = k / length;

            if (k < length)
                return Math.sqrt(vertexToCenter.x * vertexToCenter.x + vertexToCenter.y * vertexToCenter.y - k * k) <= circle.getRadius();
        }
        return false;
    }

    private boolean circleCenterInsideTriangle(Circle circle, Triangle triangle) {
        Vector vertex1To2 = triangle.getBottomRightVertex().vectorTo(triangle.getTopRightVertex());
        Vector vertex2To3 = triangle.getTopRightVertex().vectorTo(triangle.getBottomLeftVertex());
        Vector vertex3To1 = triangle.getBottomLeftVertex().vectorTo(triangle.getBottomRightVertex());
        Vector vertex1ToCenter = triangle.getBottomRightVertex().vectorTo(circle.getPosition());
        Vector vertex2ToCenter = triangle.getTopRightVertex().vectorTo(circle.getPosition());
        Vector vertex3ToCenter = triangle.getBottomLeftVertex().vectorTo(circle.getPosition());

        boolean centerInsideV1V2 = vertex1To2.y * vertex1ToCenter.x - vertex1To2.x * vertex1ToCenter.y < 0;
        boolean centerInsideV2V3 = vertex2To3.y * vertex2ToCenter.x - vertex2To3.x * vertex2ToCenter.y < 0;
        boolean centerInsideV3V1 = vertex3To1.y * vertex3ToCenter.x - vertex3To1.x * vertex3ToCenter.y < 0;

        return centerInsideV1V2 && centerInsideV2V3 && centerInsideV3V1;
    }

    private boolean circleIntersectingWithTriangleVertices(Circle circle, Triangle triangle) {
        boolean intersectingWithVertex1 = circleIntersectingWithTriangleVertex(circle, triangle.getBottomRightVertex());
        boolean intersectingWithVertex2 = circleIntersectingWithTriangleVertex(circle, triangle.getTopRightVertex());
        boolean intersectingWithVertex3 = circleIntersectingWithTriangleVertex(circle, triangle.getBottomLeftVertex());
        return intersectingWithVertex1 || intersectingWithVertex2 || intersectingWithVertex3;
    }

    private boolean circleIntersectingWithTriangleVertex(Circle circle, Vector vertex) {
        Vector centerToVertex = circle.getPosition().vectorTo(vertex);
        return centerToVertex.length() <= circle.getRadius();
    }

}
