/*
 * Created by Julian Mutter on 7/10/18 3:58 PM
 * Copyright (c) 2018. All rights reserved.
 * Last modified 7/10/18 3:54 PM
 *
 */

package de.frajul.endlessroll.entities.shapes;

import android.support.annotation.DrawableRes;
import android.support.annotation.StringRes;

import de.frajul.endlessroll.R;
import de.frajul.endlessroll.entities.textures.Texture;
import de.frajul.endlessroll.entities.textures.TexturePack;

/**
 * Created by Julian on 22.10.2017.
 */

public enum PlayerShape {
    BALL(R.string.playershape_name_ball, R.string.playershape_description_ball,
            R.drawable.playershapes_ball, true, new EmptyTask()),
    CLOCK(R.string.playershape_name_clock, R.string.playershape_description_clock,
            R.drawable.playershapes_clock, true, new CollectStarTask(15)),
    SMILEY(R.string.playershape_name_smiley, R.string.playershape_description_smiley,
            R.drawable.playershapes_smiley, true, new CollectStarTask(50)),
    WHEEL(R.string.playershape_name_wheel, R.string.playershape_description_wheel,
            R.drawable.playershapes_wheel, true, new CollectStarTask(80)),

    PACMAN(R.string.playershape_name_pacman, R.string.playershape_description_pacman,
            R.drawable.playershapes_pacman, false, new CompleteWorldTask(1)),
    SOCCER(R.string.playershape_name_soccer, R.string.playershape_description_soccer, R.drawable.playershapes_soccer, false, new CompleteWorldTask(2)),
    HYPNO_SPIRAL(R.string.playershape_name_hypno_spiral,
            R.string.playershape_description_hypno_spiral, R.drawable.playershapes_hypno_spiral, false,
            new CollectEnergyTask(5)),
    BLUE(R.string.playershape_name_blue, R.string.playershape_description_blue,
            R.drawable.playershapes_blue, false, new CollectEnergyTask(20)),
    SUN(R.string.playershape_name_sun, R.string.playershape_description_sun,
            R.drawable.playershapes_sun, false, new CollectEnergyTask(30)),
    RAINBOW(R.string.playershape_name_rainbow, R.string.playershape_description_rainbow, R.drawable.playershapes_rainbow, true, new EmptyTask());

    @StringRes
    private int nameId;
    @StringRes
    private int descriptionId;
    @DrawableRes
    private int drawableId;
    private boolean inTopRow;
    private Task unlockTask;

    private Texture texture;

    PlayerShape(@StringRes int nameId, @StringRes int descriptionId, @DrawableRes int drawableId, boolean inTopRow, Task unlockTask) {
        this.nameId = nameId;
        this.descriptionId = descriptionId;
        this.drawableId = drawableId;
        this.inTopRow = inTopRow;
        this.unlockTask = unlockTask;
    }

    public static void loadAllTextures(TexturePack texturePack) {
        for (PlayerShape playerShape : values())
            playerShape.loadTexture(texturePack);
    }

    private void loadTexture(TexturePack texturePack) {
        if (drawableId == -1)
            return;
        texture = texturePack.loadTexture(drawableId);
    }

    public Task getUnlockTask() {
        return unlockTask;
    }

    @StringRes
    public int getNameId() {
        return nameId;
    }

    @StringRes
    public int getDescriptionId() {
        return descriptionId;
    }

    @DrawableRes
    public int getDrawableId() {
        return drawableId;
    }

    public Texture getTexture() {
        return texture;
    }

    public boolean isInTopRow() {
        return inTopRow;
    }
}
