/*
 * Created by Julian Mutter on 7/10/18 3:58 PM
 * Copyright (c) 2018. All rights reserved.
 * Last modified 7/10/18 3:54 PM
 *
 */

package de.frajul.endlessroll.entities.tools;

import de.frajul.endlessroll.data.Vector;
import de.frajul.endlessroll.entities.DestroyEffect;
import de.frajul.endlessroll.entities.Player;
import de.frajul.endlessroll.entities.collision.geometry.Circle;
import de.frajul.endlessroll.entities.collision.geometry.Geometry;
import de.frajul.endlessroll.entities.particles.ParticleSource;
import de.frajul.endlessroll.entities.particles.ParticleSystem;
import de.frajul.endlessroll.main.game.Timer;
import de.frajul.endlessroll.sounds.SoundManager;
import de.frajul.endlessroll.sounds.SoundStream;

/**
 * Created by Julian on 11.02.2016.
 */
public class Magnet extends Tool {

    private SoundManager soundManager;
    private ParticleSource particleSource;
    private SoundStream soundStream;

    public Magnet(Vector position, ParticleSystem particleSystem, SoundManager soundManager) {
        super(ToolType.MAGNET, position, .24f, .24f, false, false);
        this.soundManager = soundManager;
        animation.setRequiredDelta(300);
        animation.setIndexSequence(new int[]{1, 1, 0});
        animation.setLooping(true);
        super.setFloating(true);
        particleSource = new ParticleSource(new Vector(position), particleSystem.magnet);
        particleSource.start();
        soundStream = soundManager.playSound(soundManager.magnetSound);
    }

    @Override
    public void destroy(DestroyEffect destroyEffect) {
        super.destroy(destroyEffect);
        particleSource.kill();
        soundManager.stopSound(soundStream);
    }

    public void updateSoundVolume(Player player) {
        float distanceToPlayer = super.getPosition().vectorTo(player.getPosition()).length();
        float factor = 1.0f / distanceToPlayer;
        if (distanceToPlayer > 3 && player.getPosition().getX() > super.getPosition().getX()) {
            soundManager.stopSound(soundStream);
            return;
        }
        soundStream.setCurrentVolumeModifier(factor);
        soundManager.onStreamVolumeChanged(soundStream);
    }

    @Override
    public void onPlayerCollision(Player player, Timer timer) {
        float fromPlayerDistance = player.getPosition().vectorTo(super.getPosition()).length();
        float fromPlayerDistanceGreaterFour = Math.max(fromPlayerDistance, 0.4f);
        float fromPlayerYDistance = super.getPosition().y - player.getPosition().y;
        float force = 0.0000012f / (fromPlayerDistanceGreaterFour * fromPlayerDistanceGreaterFour);
        force *= ToolType.MAGNET.getCurrentUpgradeValue(ToolUpgradeType.FORCE) / 100;
        force *= timer.getFrameTimeMilliseconds();

        if (fromPlayerYDistance < 0) {
            force = -force;
        }
        player.addForce(force);
    }

    @Override
    protected Geometry createWorldCollisionBounds() {
        return this;
    }

    @Override
    protected Geometry createPlayerCollisionBounds() {
        return new Circle(super.getPosition(), 2);
    }
}
