/*
 * Created by Julian Mutter on 7/10/18 3:58 PM
 * Copyright (c) 2018. All rights reserved.
 * Last modified 7/10/18 3:54 PM
 *
 */

package de.frajul.endlessroll.main.screens;

import android.view.View;
import android.widget.LinearLayout;

import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdView;

import java.util.concurrent.atomic.AtomicBoolean;

import de.frajul.endlessroll.R;
import de.frajul.endlessroll.levels.Level;
import de.frajul.endlessroll.levels.LevelPack;
import de.frajul.endlessroll.main.GameActivity;
import de.frajul.endlessroll.main.GameLog;
import de.frajul.endlessroll.main.ads.AdLoadingListener;
import de.frajul.endlessroll.main.ads.CustomAdListener;
import de.frajul.endlessroll.views.LevelButton;
import de.frajul.endlessroll.views.LevelButtonOnClickListener;
import de.frajul.endlessroll.views.TopBar;

/**
 * Created by Julian on 23.04.2016.
 */
public class LevelsScreen extends Screen<LinearLayout> implements LevelButtonOnClickListener, AdLoadingListener {

    private LevelPack levelPack;

    private TopBar topBar;
    private LinearLayout topRow;
    private LinearLayout bottomRow;
    private AdView adView;
    private AtomicBoolean waitingForAdResponse;
    private AtomicBoolean adLoaded;

    public LevelsScreen(GameActivity gameActivity) {
        super(ScreenType.LEVELS, gameActivity, R.layout.levels);
        topBar = super.createTopBar(R.id.levels_topbar);
        topRow = layout.findViewById(R.id.levels_top_row);
        bottomRow = layout.findViewById(R.id.levels_bottom_row);
        adView = layout.findViewById(R.id.levels_ad_view);
        adView.setAdListener(new CustomAdListener(this));
        adView.setVisibility(View.GONE);

        waitingForAdResponse = new AtomicBoolean(false);
        adLoaded = new AtomicBoolean(false);
        loadAd();
    }

    public void onLevelPackSelected(LevelPack levelPack) {
        this.levelPack = levelPack;
    }

    private void build() {
        topRow.removeAllViews();
        bottomRow.removeAllViews();
        int levelCount = levelPack.getLevels().size();

        LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(
                LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT);
        params.setMargins(15, 15, 15, 15);

        for (Level level : levelPack.getLevels())
            createButton(level, levelCount, params);
    }

    private void createButton(Level level, int levelCount, LinearLayout.LayoutParams params) {
        LevelButton button = new LevelButton(gameActivity, this, R.layout.levelbutton);
        button.init(level);

        int halfLevelCount = levelCount / 2;
        if (levelCount % 2 == 1)
            halfLevelCount++;

        if (level.getId() <= halfLevelCount)
            topRow.addView(button.getView(), params);
        else
            bottomRow.addView(button.getView(), params);
    }

    private void loadAd() {
        waitingForAdResponse.set(true);
        AdRequest adRequest = new AdRequest.Builder().build();
        adView.loadAd(adRequest);
    }

    @Override
    public void prepareToBeShown() {
        gameActivity.getSoundManager().menuMusic.start();

        if (!adLoaded.get() && !waitingForAdResponse.get())
            loadAd();

        topBar.update();
        build();
    }

    @Override
    public void onBackKeyDown() {
        flipTo(ScreenType.WORLDS);
    }

    @Override
    public void onClick(LevelButton levelButton) {
        Level level = levelButton.getLevel();
        if (!level.isLocked())
            gameActivity.startGame(levelPack, level);
    }

    @Override
    public void onAdLoaded() {
        adLoaded.set(true);
        waitingForAdResponse.set(false);
        adView.setVisibility(View.VISIBLE);
    }

    @Override
    public void onAdFailedToLoad(int errorCode) {
        adLoaded.set(false);
        waitingForAdResponse.set(false);
        GameLog.i("LevelsScreen-Ad failed to load. ErrorCode=" + errorCode);
        adView.setVisibility(View.GONE);
    }
}
