/*
 * Created by Julian Mutter on 7/10/18 3:58 PM
 * Copyright (c) 2018. All rights reserved.
 * Last modified 7/10/18 3:54 PM
 *
 */

package de.frajul.endlessroll.main.screens;

import android.graphics.Typeface;
import android.support.annotation.IdRes;
import android.view.View;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.RelativeLayout;
import android.widget.TextView;

import de.frajul.endlessroll.R;
import de.frajul.endlessroll.main.GameActivity;
import de.frajul.endlessroll.main.MyGlSurfaceView;
import de.frajul.endlessroll.main.game.StartScene;
import de.frajul.endlessroll.sqlDatabase.MyDatabase;
import de.frajul.endlessroll.views.ConfirmDialog;
import de.frajul.endlessroll.views.DevToolsDialog;

/**
 * Created by Julian on 07.07.2016.
 */
public class StartScreen extends GLScreen<RelativeLayout> implements View.OnClickListener, ConfirmDialog.ConfirmDialogListener {

    private final int CALL_ID_CONFIRM_EXIT = 1;

    private StartScreenRendering rendering;
    private Button play;
    private Button showDevToolsDialog;
    private DevToolsDialog devToolsDialog;
    private ConfirmDialog confirmDialog;

    public StartScreen(GameActivity gameActivity, MyGlSurfaceView glSurfaceView) throws Exception {
        super(ScreenType.START, gameActivity, R.layout.start_screen, glSurfaceView);

        Typeface typeface = gameActivity.getTypeface();
        TextView title = layout.findViewById(R.id.startscreen_title);
        title.setTypeface(typeface);
        play = createButton(R.id.startscreen_play, typeface);
        play.startAnimation(AnimationUtils.loadAnimation(gameActivity, R.anim.rotate));
        showDevToolsDialog = createButton(R.id.startscreen_show_dev_tools_dialog, typeface);
        showDevToolsDialog.setVisibility(GameActivity.IS_DEVELOPER ? View.VISIBLE : View.GONE);

        devToolsDialog = new DevToolsDialog(gameActivity);
        confirmDialog = new ConfirmDialog(gameActivity, this);

        rendering = new StartScreenRendering(gameActivity, gameRenderer);
    }

    private Button createButton(@IdRes int id, Typeface typeface) {
        Button button = layout.findViewById(id);
        button.setTypeface(typeface);
        button.setOnClickListener(this);
        return button;
    }

    @Override
    public void prepareToBeShown() {
        ((StartScene) rendering.getScene()).randomizeWorld();
        ((StartScene) rendering.getScene()).loadTexturesForCurrentWorld();
        gameRenderer.setCurrentRendering(rendering);

        gameActivity.getSoundManager().menuMusic.start();
    }

    @Override
    public void onBackKeyDown() {
        confirmDialog.show(CALL_ID_CONFIRM_EXIT, R.string.confirm_dialog_exit);
    }

    @Override
    public void onClick(View v) {
        if (v.equals(play)) {
            gameActivity.flipToScreen(ScreenType.WORLDS);
            gameRenderer.setCurrentRendering(null);
        } if(v.equals(showDevToolsDialog)){
            devToolsDialog.show();
        }
    }


    @Override
    public void onConfirmed(int callId) {
        if (callId == CALL_ID_CONFIRM_EXIT) {
            gameActivity.exitGame();
        }
    }
}
