/*
 * Created by Julian Mutter on 9/7/18 12:36 PM
 * Copyright (c) 2018. All rights reserved.
 * Last modified 7/10/18 3:58 PM
 *
 */

package de.frajul.endlessroll.views;

import android.animation.ObjectAnimator;
import android.app.Dialog;
import android.os.Bundle;
import android.support.annotation.IdRes;
import android.view.MotionEvent;
import android.view.View;
import android.view.Window;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.view.animation.LinearInterpolator;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ScrollView;
import android.widget.TextView;

import java.util.concurrent.locks.ReentrantLock;

import de.frajul.endlessroll.R;
import de.frajul.endlessroll.main.GameActivity;
import de.frajul.endlessroll.main.screens.Screen;
import de.frajul.endlessroll.sqlDatabase.MyDatabase;

/**
 * Created by Julian on 31.10.2017.
 */

public class DevToolsDialog extends Dialog implements View.OnClickListener, ConfirmDialog.ConfirmDialogListener{

    private GameActivity gameActivity;

    private Button unlockLevels;
    private Button gain90EP;
    private Button finishFirstWorld;
    private Button toGlTestScreen;

    private ConfirmDialog confirmDialog;

    public DevToolsDialog(GameActivity gameActivity) {
        super(gameActivity);
        this.gameActivity = gameActivity;
        confirmDialog = new ConfirmDialog(gameActivity, this);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.dev_tools_dialog);

        unlockLevels = findViewById(R.id.startscreen_unlock_levels);
        unlockLevels.setOnClickListener(this);
        gain90EP = findViewById(R.id.startscreen_gain_90_ep);
        gain90EP.setOnClickListener(this);
        finishFirstWorld = findViewById(R.id.startscreen_finish_world_1);
        finishFirstWorld.setOnClickListener(this);
        toGlTestScreen = findViewById(R.id.startscreen_to_gl_test_screen);
        toGlTestScreen.setOnClickListener(this);
    }

    @Override
    public void onClick(View view) {
        if (view.equals(gain90EP)) {
            gameActivity.getUser().gainEp(90, true);
        } else if (view.equals(unlockLevels)) {
            gameActivity.getLevelManager().unlockAllLevels();
            gameActivity.getLevelManager().unlockAllPacks();
            MyDatabase database = gameActivity.getDataStorageHandler().getDatabase();
            database.open();
            database.writeAllLevelAndPackStatus(gameActivity.getLevelManager());
            database.close();
        } else if (view.equals(finishFirstWorld)) {
            confirmDialog
                    .show(1, R.string.confirm_dialog_finish_world_1);
        } else if (view.equals(toGlTestScreen)) {
            super.cancel();
            gameActivity.flipToScreen(Screen.ScreenType.GL_TEST);
        }
    }

    @Override
    public void onConfirmed(int callId) {
        gameActivity.resetData();
        gameActivity.getUser().gainEp(320, false);
        gameActivity.getLevelManager().finishFirstPack();
        gameActivity.showNewFinishedPlayerShapeTasks();
        gameActivity.getTutorialManager().getToolShopTutorial().setFinished(true);

        gameActivity.getDataStorageHandler().writeUserData(gameActivity.getUser());
        gameActivity.getDataStorageHandler().writeToolShopTutorialFinished(true);
        MyDatabase database = gameActivity.getDataStorageHandler().getDatabase();
        database.open();
        database.writeAllLevelAndPackStatus(gameActivity.getLevelManager());
        database.close();

        super.cancel();
    }
}
